@@ -53,16 +53,18 @@ namespace Banshee.Base
             return result;
         }
         
-        public static string MakePathRelativeToLibrary (string path)
+        public static string MakePathRelative (string path, string to)
         {
-            string library_location = LibraryLocation; // TODO: Use CachedLibraryLocation?
-            
-            if (path.Length < library_location.Length + 1) {
+            if (String.IsNullOrEmpty (path) || String.IsNullOrEmpty (to)) {
                 return null;
             }
             
-            return path.StartsWith (library_location)
-                ? path.Substring (library_location.Length + 1)
+            if (path.Length < to.Length + 1) {
+                return null;
+            }
+            
+            return path.StartsWith (to)
+                ? path.Substring (to.Length + 1)
                 : null;
         }
         
@@ -142,6 +144,11 @@ namespace Banshee.Base
                 if (installed_application_prefix == null) {
                     installed_application_prefix = Path.GetDirectoryName (
                         System.Reflection.Assembly.GetEntryAssembly ().Location);
+                    
+                    if (Directory.Exists (Paths.Combine (installed_application_prefix, "share", "banshee-1"))) {
+                        return installed_application_prefix;
+                    }
+                        
                     DirectoryInfo entry_directory = new DirectoryInfo (installed_application_prefix);
                     
                     if (entry_directory != null && entry_directory.Parent != null && entry_directory.Parent.Parent != null) {
@@ -160,5 +167,10 @@ namespace Banshee.Base
         public static string InstalledApplicationData {
             get { return Path.Combine (InstalledApplicationDataRoot, "banshee-1"); }
         }
+        
+        public static string GetInstalledDataDirectory (string path)
+        {
+            return Path.Combine (InstalledApplicationData, path);
+        }
     }
 }
