@@ -49,10 +49,14 @@ using Banshee.ServiceStack;
 
 namespace Banshee.Collection.Database
 {
+    public enum TrackUriType : int {
+        AbsolutePath = 0,
+        RelativePath = 1,
+        AbsoluteUri = 2
+    }
+
     public class DatabaseTrackInfo : TrackInfo, ICacheableItem
     {
-        private bool? artist_changed = null, album_changed = null;
-        
         private static BansheeModelProvider<DatabaseTrackInfo> provider = new BansheeModelProvider<DatabaseTrackInfo> (
             ServiceManager.DbConnection, "CoreTracks"
         );
@@ -61,26 +65,32 @@ namespace Banshee.Collection.Database
             get { return provider; }
         }
 
-        private enum UriType : int {
-            AbsolutePath,
-            RelativePath,
-            AbsoluteUri
-        }
+        private bool? artist_changed = null, album_changed = null;
+        private bool uri_fields_dirty = false;
         
         public DatabaseTrackInfo () : base ()
         {
         }
 
+        public DatabaseTrackInfo (DatabaseTrackInfo original) : base ()
+        {
+            Provider.Copy (original, this);
+        }
+
         public override void IncrementPlayCount ()
         {
-            base.IncrementPlayCount ();
-            Save ();
+            if (Provider.Refresh (this)) {
+                base.IncrementPlayCount ();
+                Save ();
+            }
         }
 
         public override void IncrementSkipCount ()
         {
-            base.IncrementSkipCount ();
-            Save ();
+            if (Provider.Refresh (this)) {
+                base.IncrementSkipCount ();
+                Save ();
+            }
         }
 
         public override bool TrackEqual (TrackInfo track)
@@ -98,22 +108,33 @@ namespace Banshee.Collection.Database
         {
             Save (true);
         }
+        
+        public DatabaseArtistInfo Artist {
+            get { return DatabaseArtistInfo.FindOrCreate (ArtistName); }
+        }
+        
+        public DatabaseAlbumInfo Album {
+            get { return DatabaseAlbumInfo.FindOrCreate (Artist, AlbumTitle); }
+        }
 
         public void Save (bool notify)
         {
             // If either the artist or album changed, 
             if (ArtistId == 0 || AlbumId == 0 || artist_changed == true || album_changed == true) {
-                DatabaseArtistInfo artist = DatabaseArtistInfo.FindOrCreate (ArtistName);
+                DatabaseArtistInfo artist = Artist;
                 ArtistId = artist.DbId;
            
-                DatabaseAlbumInfo album = DatabaseAlbumInfo.FindOrCreate (artist, AlbumTitle);
+                DatabaseAlbumInfo album = Album;
                 AlbumId = album.DbId;
                 
                 // TODO get rid of unused artists/albums
             }
             
             DateUpdated = DateTime.Now;
-            bool is_new = TrackId == 0;
+
+            bool is_new = (TrackId == 0);
+            if (is_new) DateAdded = DateUpdated;
+
             Provider.Save (this);
 
             if (notify) {
@@ -131,26 +152,29 @@ namespace Banshee.Collection.Database
             get { return track_id; }
         }
 
-
-        [DatabaseColumn ("PrimarySourceID", Index = "CoreTracksSourceIndex")]
         private int primary_source_id;
+        [DatabaseColumn ("PrimarySourceID")]
         public int PrimarySourceId {
             get { return primary_source_id; }
+            set {
+                primary_source_id = value;
+                UpdateUri ();
+            }
         }
 
         public PrimarySource PrimarySource {
             get { return PrimarySource.GetById (primary_source_id); }
-            set { primary_source_id = value.DbId; }
+            set { PrimarySourceId = value.DbId; }
         }
 
-        [DatabaseColumn ("ArtistID", Index = "CoreTracksArtistIndex")]
+        [DatabaseColumn ("ArtistID")]
         private int artist_id;
         public int ArtistId {
             get { return artist_id; }
             set { artist_id = value; }
         }
 
-        [DatabaseColumn ("AlbumID", Index = "CoreTracksAlbumIndex")]
+        [DatabaseColumn ("AlbumID")]
         private int album_id;
         public int AlbumId {
             get { return album_id; }
@@ -188,66 +212,54 @@ namespace Banshee.Collection.Database
             set { tag_set_id = value; }
         }
         
-        private string music_brainz_id;
-        [DatabaseColumn]
-        public string MusicBrainzID {
-            get { return music_brainz_id; }
-            set { music_brainz_id = value; }
+        [DatabaseColumn ("MusicBrainzID")]
+        public override string MusicBrainzId {
+            get { return base.MusicBrainzId; }
+            set { base.MusicBrainzId = value; }
+        }
+
+        public override SafeUri Uri {
+            get { return base.Uri; }
+            set {
+                base.Uri = value;
+                uri_fields_dirty = true;
+            }
         }
         
         private string uri_field;
         [DatabaseColumn ("Uri")]
-        private string uri {
+        private string UriField {
             get {
-                return
-                    uri_field ??
-                    (Uri == null
-                        ? null
-                        : (Paths.MakePathRelativeToLibrary (Uri.AbsolutePath) ?? Uri.AbsoluteUri)
-                    );
+                if (uri_fields_dirty) {
+                    PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
+                    uri_fields_dirty = false;
+                }
+                return uri_field;
             }
             set {
                 uri_field = value;
-                if (uri_type_field.HasValue) {
-                    SetUpUri ();
-                }
+                UpdateUri ();
             }
         }
         
-        private int? uri_type_field;
+        private bool uri_type_set;
+        private TrackUriType uri_type;
         [DatabaseColumn ("UriType")]
-        private int uri_type {
+        private TrackUriType UriType {
             get {
-                return uri_type_field.HasValue
-                    ? uri_type_field.Value
-                    : (Uri == null
-                        ? (int)UriType.RelativePath
-                        : (Paths.MakePathRelativeToLibrary (Uri.AbsolutePath) != null
-                            ? (int)UriType.RelativePath
-                            : (int)UriType.AbsoluteUri));
+                if (uri_fields_dirty) {
+                    PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
+                    uri_fields_dirty = false;
+                }
+                return uri_type;
             }
             set {
-                uri_type_field = value;
-                if (uri_field != null) {
-                    SetUpUri ();
-                }
+                uri_type = value;
+                uri_type_set = true;
+                UpdateUri ();
             }
         }
         
-        bool set_up;
-        private void SetUpUri ()
-        {
-            if (set_up) {
-                return;
-            }
-            set_up = true;
-            
-            if (uri_type_field.Value == (int)UriType.RelativePath) {
-                uri_field = System.IO.Path.Combine (Paths.LibraryLocation, uri_field);
-            }
-            Uri = new SafeUri (uri_field);
-        }
-        
         [DatabaseColumn]
         public override string MimeType {
             get { return base.MimeType; }
@@ -272,6 +284,11 @@ namespace Banshee.Collection.Database
             set { base.TrackTitle = value; }
         }
         
+        [DatabaseColumn(Select = false)]
+        protected string TitleLowered {
+            get { return TrackTitle == null ? null : TrackTitle.ToLower (); }
+        }
+        
         [DatabaseColumn]
         public override int TrackNumber {
             get { return base.TrackNumber; }
@@ -387,53 +404,48 @@ namespace Banshee.Collection.Database
             set { cache_model_id = value; }
         }
 
-
-        private static HyenaSqliteCommand check_command = new HyenaSqliteCommand (
-            "SELECT COUNT(*) FROM CoreTracks WHERE Uri = ? OR Uri = ?"
-        );
-        
-        public static bool ContainsUri (SafeUri uri)
+        private void UpdateUri ()
         {
-            string relative_path = Paths.MakePathRelativeToLibrary (uri.AbsolutePath) ?? uri.AbsoluteUri;
-            return ServiceManager.DbConnection.Query<int> (
-                check_command.ApplyValues (relative_path, uri.AbsoluteUri)) > 0;
+            if (Uri == null && uri_type_set && UriField != null && PrimarySource != null) {
+                Uri = PrimarySource.UriAndTypeToSafeUri (UriType, UriField);
+            }
         }
         
-        public SafeUri CopyToLibrary ()
+        public void CopyToLibraryIfAppropriate (bool force_copy)
         {
             SafeUri old_uri = this.Uri;
             if (old_uri == null) {
                 // Get out quick, no URI set yet.
-                return null;
+                return;
             }
             
-            SafeUri library_check = new SafeUri (Paths.LibraryLocation + Path.DirectorySeparatorChar);
-        
-            bool in_library = old_uri.ToString ().StartsWith (library_check.ToString ());
+            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocation);
             //Console.WriteLine ("{0} is{1}in library.", old_uri.ToString (), in_library ? " " : " not ");
 
-            if (!in_library && LibrarySchema.CopyOnImport.Get ()) {
-                string new_filename = FileNamePattern.BuildFull (this,
-                    Path.GetExtension (old_uri.ToString ()).Substring (1));
+            if (!in_library && (LibrarySchema.CopyOnImport.Get () || force_copy)) {
+                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()).Substring (1));
                 SafeUri new_uri = new SafeUri (new_filename);
 
                 try {
                     if (Banshee.IO.File.Exists (new_uri)) {
-                        return null;
+                        return;
                     }
                     
-                    // TODO: Once GnomeVfs and Unix have proper Copy providers, use IOProxy.File.Copy instead.
-                    System.IO.File.Copy (old_uri.LocalPath, new_uri.LocalPath);
-                    
-                    // Return new SafeUri after copy
-                    return new_uri;
+                    Banshee.IO.File.Copy (old_uri, new_uri, false);
+                    Uri = new_uri;
                 } catch (Exception e) {
                     Log.Error (String.Format("Exception copying into library: {0}", e), false);
-                    return null;
                 }
             }
-            
-            return null;
+        }
+
+        private static HyenaSqliteCommand check_command = new HyenaSqliteCommand (
+            "SELECT COUNT(*) FROM CoreTracks WHERE PrimarySourceId IN (?) AND (Uri = ? OR Uri = ?)"
+        );
+        
+        public static bool ContainsUri (SafeUri uri, string relative_path, int [] primary_sources)
+        {
+            return ServiceManager.DbConnection.Query<int> (check_command.ApplyValues (primary_sources, relative_path, uri.AbsoluteUri)) > 0;
         }
     }
 }
