@@ -45,7 +45,9 @@ using Banshee.Equalizer;
 
 namespace Banshee.MediaEngine
 {
-    public class PlayerEngineService : IService, IPlayerEngineService
+    public delegate bool TrackInterceptHandler (TrackInfo track);
+
+    public class PlayerEngineService : IInitializeService, IRequiredService, IPlayerEngineService, IDisposable
     {   
         private List<PlayerEngine> engines = new List<PlayerEngine> ();
         private PlayerEngine active_engine;
@@ -57,6 +59,7 @@ namespace Banshee.MediaEngine
         public event PlayerEngineEventHandler EventChanged;
         public event PlayerEngineStateHandler StateChanged;
         public event EventHandler PlayWhenIdleRequest;
+        public event TrackInterceptHandler TrackIntercept;
         
         private event DBusPlayerEngineEventHandler dbus_event_changed;
         event DBusPlayerEngineEventHandler IPlayerEngineService.EventChanged {
@@ -71,6 +74,10 @@ namespace Banshee.MediaEngine
         }
         
         public PlayerEngineService ()
+        {
+        }
+        
+        public void Initialize ()
         {
             preferred_engine_id = EngineSchema.Get();
             
@@ -78,7 +85,7 @@ namespace Banshee.MediaEngine
                 default_engine = engines[0];
             }
             
-            foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/PlayerEngines/PlayerEngine")) {
+            foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/MediaEngine/PlayerEngine")) {
                 LoadEngine (node);
             }
             
@@ -124,6 +131,19 @@ namespace Banshee.MediaEngine
 
         public void Dispose ()
         {
+            MetadataService.Instance.HaveResult -= OnMetadataServiceHaveResult;
+            
+            foreach (PlayerEngine engine in engines) {
+                engine.Dispose ();
+            }
+            
+            active_engine = null;
+            default_engine = null;
+            pending_engine = null;
+            
+            preferred_engine_id = null;
+            
+            engines.Clear ();
         }
         
         private void OnMetadataServiceHaveResult (object o, MetadataLookupResultArgs args)
@@ -225,9 +245,25 @@ namespace Banshee.MediaEngine
             }
         }
         
+        private bool OnTrackIntercept (TrackInfo track)
+        {
+            TrackInterceptHandler handler = TrackIntercept;
+            if (handler == null) {
+                return false;
+            }
+            
+            bool handled = false;
+            
+            foreach (TrackInterceptHandler single_handler in handler.GetInvocationList ()) {
+                handled |= single_handler (track);
+            }
+            
+            return handled;
+        }
+        
         public void Open (TrackInfo track)
         {
-            if (!track.CanPlay) {
+            if (!track.CanPlay || OnTrackIntercept (track)) {
                 return;
             }
                
@@ -246,7 +282,7 @@ namespace Banshee.MediaEngine
         
         public void OpenPlay (TrackInfo track)
         {
-            if (!track.CanPlay) {
+            if (!track.CanPlay || OnTrackIntercept (track)) {
                 return;
             }
         
@@ -278,23 +314,32 @@ namespace Banshee.MediaEngine
             } else {
                 return;
             }
+
+            IncrementLastPlayed ();
             
             FindSupportingEngine (uri);
             CheckPending ();
             
-            if (active_engine.CurrentTrack != null) {
+            if (track != null) {
+                active_engine.Open (track);
+                incremented_last_played = false;
+            } else if (uri != null) {
+                active_engine.Open (uri);
+                incremented_last_played = false;
+            }
+        }
+
+        private bool incremented_last_played = true;
+        public void IncrementLastPlayed ()
+        {
+            if (!incremented_last_played && active_engine.CurrentTrack != null) {
                 // If we're at least 50% done playing a song, mark it as played, otherwise as skipped
                 if (active_engine.Length > 0 && active_engine.Position >= active_engine.Length / 2) {
                     active_engine.CurrentTrack.IncrementPlayCount ();
                 } else {
                     active_engine.CurrentTrack.IncrementSkipCount ();
                 }
-            }
-            
-            if (track != null) {
-                active_engine.Open (track);
-            } else if (uri != null) {
-                active_engine.Open (uri);
+                incremented_last_played = true;
             }
         }
         
@@ -330,8 +375,14 @@ namespace Banshee.MediaEngine
         
         public void Close ()
         {
+            Close (false);
+        }
+        
+        public void Close (bool fullShutdown)
+        {
+            IncrementLastPlayed ();
             active_engine.Reset ();
-            active_engine.Close ();
+            active_engine.Close (fullShutdown);
         }
         
         public void Play ()
