@@ -39,7 +39,7 @@ using Banshee.MediaEngine;
 
 namespace Banshee.PlaybackController
 {
-    public class PlaybackControllerService : IService, ICanonicalPlaybackController, IPlaybackControllerExportable
+    public class PlaybackControllerService : IRequiredService, ICanonicalPlaybackController, IPlaybackControllerExportable
     {
         private enum Direction
         {
@@ -55,7 +55,7 @@ namespace Banshee.PlaybackController
         private bool raise_started_after_transition = false;
         private bool transition_track_started = false;
         
-        private Random random = new Random ();
+        //private Random random = new Random ();
     
         private PlaybackShuffleMode shuffle_mode;
         private PlaybackRepeatMode repeat_mode;
@@ -100,7 +100,11 @@ namespace Banshee.PlaybackController
             switch (args.Event) {
                 case PlayerEngineEvent.EndOfStream:
                     if (!StopWhenFinished) {
-                        Next ();
+                        if (RepeatMode == PlaybackRepeatMode.RepeatSingle) {
+                            QueuePlayTrack ();
+                        } else {
+                            Next ();
+                        }
                     } else {
                         OnStopped ();
                     }
@@ -127,7 +131,8 @@ namespace Banshee.PlaybackController
         
         public void First ()
         {
-            raise_started_after_transition = true;
+            // This and OnTransition() below commented out b/c of BGO #524556
+            //raise_started_after_transition = true;
             
             if (Source is IBasicPlaybackController) {
                 ((IBasicPlaybackController)Source).First ();
@@ -135,30 +140,44 @@ namespace Banshee.PlaybackController
                 ((ICanonicalPlaybackController)this).First ();
             }
             
-            OnTransition ();
+            //OnTransition ();
         }
         
         public void Next ()
+        {
+            Next (RepeatMode == PlaybackRepeatMode.RepeatAll);
+        }
+        
+        public void Next (bool restart)
         {
             raise_started_after_transition = true;
+
+            player_engine.IncrementLastPlayed ();
             
             if (Source is IBasicPlaybackController) {
-                ((IBasicPlaybackController)Source).Next ();
+                ((IBasicPlaybackController)Source).Next (restart);
             } else {
-                ((ICanonicalPlaybackController)this).Next ();
+                ((ICanonicalPlaybackController)this).Next (restart);
             }
             
             OnTransition ();
         }
-        
+
         public void Previous ()
+        {
+            Previous (RepeatMode == PlaybackRepeatMode.RepeatAll);
+        }
+        
+        public void Previous (bool restart)
         {
             raise_started_after_transition = true;
+
+            player_engine.IncrementLastPlayed ();
             
             if (Source is IBasicPlaybackController) {
-                ((IBasicPlaybackController)Source).Previous ();
+                ((IBasicPlaybackController)Source).Previous (restart);
             } else {
-                ((ICanonicalPlaybackController)this).Previous ();
+                ((ICanonicalPlaybackController)this).Previous (restart);
             }
             
             OnTransition ();
@@ -171,7 +190,7 @@ namespace Banshee.PlaybackController
             }
         }
         
-        void ICanonicalPlaybackController.Next ()
+        void ICanonicalPlaybackController.Next (bool restart)
         {
             TrackInfo tmp_track = CurrentTrack;
 
@@ -181,7 +200,7 @@ namespace Banshee.PlaybackController
                     previous_stack.Push (tmp_track);
                 }
             } else {
-                TrackInfo next_track = QueryTrack (Direction.Next);
+                TrackInfo next_track = QueryTrack (Direction.Next, restart);
                 if (next_track != null) {
                     if (tmp_track != null) {
                         previous_stack.Push (tmp_track);
@@ -196,7 +215,7 @@ namespace Banshee.PlaybackController
             QueuePlayTrack ();
         }
         
-        void ICanonicalPlaybackController.Previous ()
+        void ICanonicalPlaybackController.Previous (bool restart)
         {
             if (CurrentTrack != null && previous_stack.Count > 0) {
                 next_stack.Push (current_track);
@@ -205,30 +224,51 @@ namespace Banshee.PlaybackController
             if (previous_stack.Count > 0) {
                 CurrentTrack = previous_stack.Pop ();
             } else {
-                CurrentTrack = QueryTrack (Direction.Previous);
+                TrackInfo track = CurrentTrack = QueryTrack (Direction.Previous, restart);
+                if (track != null) {
+                    CurrentTrack = track;
+                } else {
+                    return;
+                }
             }
             
             QueuePlayTrack ();
         }
         
-        private TrackInfo QueryTrack (Direction direction)
+        private TrackInfo QueryTrack (Direction direction, bool restart)
         {
             Log.DebugFormat ("Querying model for track to play in {0}:{1} mode", ShuffleMode, direction);
             return ShuffleMode == PlaybackShuffleMode.Linear
-                ? QueryTrackLinear (direction)
-                : QueryTrackRandom ();
+                ? QueryTrackLinear (direction, restart)
+                : QueryTrackRandom (restart);
         }
         
-        private TrackInfo QueryTrackLinear (Direction direction)
+        private TrackInfo QueryTrackLinear (Direction direction, bool restart)
         {
+            if (Source.TrackModel.Count == 0)
+                return null;
+
             int index = Source.TrackModel.IndexOf (CurrentTrack);
-            return Source.TrackModel[index < 0 ? 0 : index + (direction == Direction.Next ? 1 : -1)];
+            if (index == -1) {
+                return Source.TrackModel[0];
+            } else {
+                index += (direction == Direction.Next ? 1 : -1);
+                if (index >= 0 && index < Source.TrackModel.Count) {
+                    return Source.TrackModel[index];
+                } else if (!restart) {
+                    return null;
+                } else if (index < 0) {
+                    return Source.TrackModel[Source.TrackModel.Count - 1];
+                } else {
+                    return Source.TrackModel[0];
+                }
+            }
         }
         
-        private TrackInfo QueryTrackRandom ()
+        private TrackInfo QueryTrackRandom (bool restart)
         {
-            // TODO let the TrackModel give us a random track
-            return Source.TrackModel[random.Next (0, Source.TrackModel.Count - 1)];
+            return Source.TrackModel.GetRandom (source_set_at, restart);
+            //return Source.TrackModel[random.Next (0, Source.TrackModel.Count - 1)];
         }
         
         private void QueuePlayTrack ()
@@ -286,6 +326,7 @@ namespace Banshee.PlaybackController
             protected set { current_track = value; }
         }
         
+        protected DateTime source_set_at;
         public ITrackModelSource Source {
             get { 
                 if (source == null && ServiceManager.SourceManager.DefaultSource is ITrackModelSource) {
@@ -297,6 +338,7 @@ namespace Banshee.PlaybackController
             set {
                 if (source != value) {
                     source = value;
+                    source_set_at = DateTime.Now;
                     OnSourceChanged ();
                 }
             }
