@@ -34,6 +34,7 @@ using System.Collections.Generic;
 
 using Mono.Unix;
 
+using Hyena;
 using Hyena.Data;
 using Hyena.Data.Sqlite;
 using Hyena.Collections;
@@ -116,28 +117,7 @@ namespace Banshee.Playlist
 
         public override bool AcceptsInputFromSource (Source source)
         {
-            // TODO: Probably should be more restrictive than this
-            return source is DatabaseSource;
-        }
-        
-        public override void MergeSourceInput (Source from, SourceMergeType mergeType)
-        {
-            DatabaseSource source = from as DatabaseSource;
-            if (source == null || !(source.TrackModel is TrackListDatabaseModel)) {
-                return;
-            }
-            
-            TrackListDatabaseModel model = (TrackListDatabaseModel)source.TrackModel;
-            
-            switch (mergeType) {
-                case SourceMergeType.ModelSelection:
-                    AddSelectedTracks (model);
-                    break;
-                case SourceMergeType.Source:
-                    AddTrackRange (model, new RangeCollection.Range (0, model.Count));
-                    Reload ();
-                    break;
-            }
+            return base.AcceptsInputFromSource (source) && (Parent == null || source == Parent || source.Parent == Parent);
         }
         
         public override SourceMergeType SupportedMergeTypes {
@@ -183,10 +163,20 @@ namespace Banshee.Playlist
             }
         }
 
+        // We can add tracks only if our parent can
+        public override bool CanAddTracks {
+            get {
+                DatabaseSource ds = Parent as DatabaseSource;
+                return ds != null ? ds.CanAddTracks : base.CanAddTracks;
+            }
+        }
+
         // Have our parent handle deleting tracks
-        public override void DeleteSelectedTracks (TrackListDatabaseModel model)
+        public override void DeleteSelectedTracks ()
         {
-            (Parent as DatabaseSource).DeleteSelectedTracks (model);
+            if (Parent is PrimarySource) {
+                (Parent as PrimarySource).DeleteSelectedTracksFromChild (this);
+            }
         }
 
 #endregion
@@ -226,19 +216,23 @@ namespace Banshee.Playlist
             Reload ();
         }*/
         
-        public virtual void AddSelectedTracks (TrackListDatabaseModel from)
+        public override bool AddSelectedTracks (Source source)
         {
-            if (from == track_model)
-                return;
-
-            WithTrackSelection (from, AddTrackRange);
-            Reload ();
-            OnUserNotifyUpdated ();
+            if (Parent == null || source == Parent || source.Parent == Parent) {
+                return base.AddSelectedTracks (source);
+            } else {
+                // Adding from a different primary source, so add to our primary source first
+                //PrimarySource primary = Parent as PrimarySource;
+                //primary.AddSelectedTracks (model);
+                // then add to us
+                Log.Information ("Note: Feature Not Implemented", String.Format ("In this alpha release, you can only add tracks to {0} from {1} or its playlists.", Name, Parent.Name), true);
+            }
+            return false;
         }
 
-        TrackListDatabaseModel last_add_range_from_model;
+        DatabaseTrackListModel last_add_range_from_model;
         HyenaSqliteCommand last_add_range_command = null;
-        protected virtual void AddTrackRange (TrackListDatabaseModel from, RangeCollection.Range range)
+        protected override void AddTrackRange (DatabaseTrackListModel from, RangeCollection.Range range)
         {
             last_add_range_command = (!from.CachesJoinTableEntries)
                 ? add_track_range_command
@@ -246,15 +240,15 @@ namespace Banshee.Playlist
                     ? last_add_range_command
                     : new HyenaSqliteCommand (String.Format (add_track_range_from_joined_model_sql, from.JoinTable, from.JoinPrimaryKey));
 
-            last_add_range_command.ApplyValues (DbId, from.CacheId, range.Start, range.End - range.Start + 1);
+            last_add_range_command.ApplyValues (DbId, from.CacheId, range.Start, range.Count);
             ServiceManager.DbConnection.Execute (last_add_range_command);
 
             last_add_range_from_model = from;
         }
 
-        protected override void RemoveTrackRange (TrackListDatabaseModel from, RangeCollection.Range range)
+        protected override void RemoveTrackRange (DatabaseTrackListModel from, RangeCollection.Range range)
         {
-            remove_track_range_command.ApplyValues (DbId, from.CacheId, range.Start, range.End - range.Start + 1);
+            remove_track_range_command.ApplyValues (DbId, from.CacheId, range.Start, range.Count);
             ServiceManager.DbConnection.Execute (remove_track_range_command);
         }
 
