@@ -48,10 +48,20 @@ namespace Banshee.Collection.Gui
             ColumnController = column_controller;
             
             RowActivated += delegate {
-                ServiceManager.PlaybackController.First ();
+                ServiceManager.PlaybackController.Source = (ServiceManager.SourceManager.ActiveSource as Banshee.Sources.ITrackModelSource);
+                ServiceManager.PlaybackController.Next ();
             };
         }
 
+        // TODO add context menu for artists/albums...probably need a Banshee.Gui/ArtistActions.cs file.  Should
+        // make TrackActions.cs more generic with regards to the TrackSelection stuff, using the new properties
+        // set on the sources themselves that give us access to the IListView<T>.
+        /*protected override bool OnPopupMenu ()
+        {
+            ServiceManager.Get<InterfaceActionService> ().TrackActions["TrackContextMenuAction"].Activate ();
+            return true;
+        }*/
+
         protected override bool OnFocusInEvent(Gdk.EventFocus evnt)
         {
             ServiceManager.Get<InterfaceActionService> ().TrackActions.SuppressSelectActions ();
