@@ -30,8 +30,12 @@ using System;
 using Mono.Unix;
 using Gtk;
 
+using Hyena.Data.Gui;
+
 using Banshee.Base;
 using Banshee.Collection;
+using Banshee.Sources;
+using Banshee.Configuration;
 using Banshee.ServiceStack;
 using Banshee.MediaEngine;
 using Banshee.PlaybackController;
@@ -44,31 +48,38 @@ namespace Banshee.Gui
         private InterfaceActionService action_service;
         private Gtk.Action play_pause_action;
         private PlaybackRepeatActions repeat_actions;
+        private PlaybackShuffleActions shuffle_actions;
 
         public PlaybackRepeatActions RepeatActions {
             get { return repeat_actions; }
         }
         
+        public PlaybackShuffleActions ShuffleActions {
+            get { return shuffle_actions; }
+        }
+        
         public PlaybackActions (InterfaceActionService actionService) : base ("Playback")
         {
-            repeat_actions = new PlaybackRepeatActions (actionService);
-
             Add (new ActionEntry [] {
-                new ActionEntry ("PlayPauseAction", "media-playback-start",
+                new ActionEntry ("PlayPauseAction", null,
                     Catalog.GetString ("_Play"), "space",
                     Catalog.GetString ("Play or pause the current item"), OnPlayPauseAction),
                     
-                new ActionEntry ("NextAction", "media-skip-forward",
+                new ActionEntry ("NextAction", null,
                     Catalog.GetString ("_Next"), "N",
                     Catalog.GetString ("Play the next item"), OnNextAction),
                     
-                new ActionEntry ("PreviousAction", "media-skip-backward",
+                new ActionEntry ("PreviousAction", null,
                     Catalog.GetString ("Pre_vious"), "B",
                     Catalog.GetString ("Play the previous item"), OnPreviousAction),
 
                 new ActionEntry ("SeekToAction", null,
                     Catalog.GetString ("Seek _to..."), "T",
                     Catalog.GetString ("Seek to a specific location in current item"), OnSeekToAction),
+
+                new ActionEntry ("JumpToPlayingTrackAction", null,
+                    Catalog.GetString("_Jump to Playing Song"), "<control>J",
+                    Catalog.GetString ("Jump to the currently playing item"), OnJumpToPlayingTrack),
                 
                 new ActionEntry ("RestartSongAction", null,
                     Catalog.GetString ("_Restart Song"), "R",
@@ -76,27 +87,33 @@ namespace Banshee.Gui
             });
             
             Add (new ToggleActionEntry [] {
-                new ToggleActionEntry ("ShuffleAction", "media-playlist-shuffle",
-                    Catalog.GetString ("Shu_ffle"), null,
-                    Catalog.GetString ("Toggle between shuffle or continuous playback modes"), 
-                    OnShuffleAction, false),
-                    
                 new ToggleActionEntry ("StopWhenFinishedAction", null,
                     Catalog.GetString ("_Stop When Finished"), "<Shift>space",
                     Catalog.GetString ("Stop playback after the current item finishes playing"), 
                     OnStopWhenFinishedAction, false)
             });
-
+            
             actionService.GlobalActions.Add (new ActionEntry [] {
                 new ActionEntry ("PlaybackMenuAction", null,
                     Catalog.GetString ("_Playback"), null, null, null),
             });
+
+            this["JumpToPlayingTrackAction"].Sensitive = false;
+            this["RestartSongAction"].Sensitive = false;
+            this["SeekToAction"].Sensitive = false;
             
+            this["PlayPauseAction"].IconName = "media-playback-start";
+            this["NextAction"].IconName = "media-skip-forward";
+            this["PreviousAction"].IconName = "media-skip-backward";
+            
+            action_service = actionService;
             ServiceManager.PlayerEngine.StateChanged += OnPlayerEngineStateChanged;
             ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
-            action_service = actionService;
-        }
             
+            repeat_actions = new PlaybackRepeatActions (actionService);
+            shuffle_actions = new PlaybackShuffleActions (actionService, this);
+        }
+        
         private void OnPlayerEngineStateChanged (object o, PlayerEngineStateArgs args)
         {
             if (play_pause_action == null) {
@@ -124,13 +141,25 @@ namespace Banshee.Gui
             switch (args.Event) {
                 case PlayerEngineEvent.StartOfStream:
                     TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack; 
-                    action_service["Playback.RestartSongAction"].Sensitive = !track.IsLive;
-                    action_service["Playback.RestartSongAction"].Label = (track.MediaAttributes & TrackMediaAttributes.VideoStream) == 0
+                    this["SeekToAction"].Sensitive = !track.IsLive;
+                    this["RestartSongAction"].Sensitive = !track.IsLive;
+                    this["RestartSongAction"].Label = (track.MediaAttributes & TrackMediaAttributes.VideoStream) == 0
                         ? Catalog.GetString ("_Restart Song")
                         : Catalog.GetString ("_Restart Video");
+
+                    this["JumpToPlayingTrackAction"].Sensitive = true;
+                    this["JumpToPlayingTrackAction"].Label = (track.MediaAttributes & TrackMediaAttributes.VideoStream) == 0
+                        ? Catalog.GetString ("_Jump to Playing Song")
+                        : Catalog.GetString ("_Jump to Playing Video");
                     break;
+
+                case PlayerEngineEvent.Error:
                 case PlayerEngineEvent.EndOfStream:
-                    ToggleAction stop_action = (ToggleAction)action_service["Playback.StopWhenFinishedAction"];
+                    this["JumpToPlayingTrackAction"].Sensitive = false;
+                    this["RestartSongAction"].Sensitive = false;
+                    this["SeekToAction"].Sensitive = false;
+
+                    ToggleAction stop_action = (ToggleAction) this["StopWhenFinishedAction"];
                     // Kinda lame, but we don't want to actually reset StopWhenFinished inside the controller
                     // since it is also listening to EOS and needs to actually stop playback; we listen here
                     // just to keep the UI in sync.
@@ -153,13 +182,13 @@ namespace Banshee.Gui
         private void ShowPause ()
         {
             play_pause_action.Label = Catalog.GetString ("_Pause");
-            play_pause_action.StockId = "media-playback-pause";
+            play_pause_action.IconName = "media-playback-pause";
         }
         
         private void ShowPlay ()
         {
             play_pause_action.Label = Catalog.GetString ("_Play");
-            play_pause_action.StockId = "media-playback-start";
+            play_pause_action.IconName = "media-playback-start";
         }
                 
         private void OnPlayPauseAction (object o, EventArgs args)
@@ -187,20 +216,36 @@ namespace Banshee.Gui
         private void OnRestartSongAction (object o, EventArgs args)
         {
             TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack;
-            ServiceManager.PlayerEngine.Close ();
-            ServiceManager.PlayerEngine.OpenPlay (track);
+            if (track != null) {
+                ServiceManager.PlayerEngine.Close ();
+                ServiceManager.PlayerEngine.OpenPlay (track);
+            }
         }
         
         private void OnStopWhenFinishedAction (object o, EventArgs args)
         {
             ServiceManager.PlaybackController.StopWhenFinished = ((ToggleAction)o).Active;
         }
-        
-        private void OnShuffleAction (object o, EventArgs args)
+
+        private void OnJumpToPlayingTrack (object o, EventArgs args)
         {
-            ServiceManager.PlaybackController.ShuffleMode = ((ToggleAction)o).Active 
-                ? PlaybackShuffleMode.Shuffle
-                : PlaybackShuffleMode.Linear;
+            Source playback_src = ServiceManager.PlaybackController.Source as Source;
+
+            if (playback_src != null && playback_src is ITrackModelSource) {
+                ITrackModelSource track_src = (playback_src as ITrackModelSource);
+                int i = track_src.TrackModel.IndexOf (ServiceManager.PlaybackController.CurrentTrack);
+                if (i != -1) {
+                    // TODO clear the search/filters if there are any, since they might be hiding the currently playing item?
+                    // and/or switch to the track's primary source?  what if it's been removed from the library all together?
+                    IListView<TrackInfo> track_list = (track_src as Source).Properties.Get<IListView<TrackInfo>> ("Track.IListView");
+                    if (track_list != null) {
+                        ServiceManager.SourceManager.SetActiveSource (playback_src);
+                        track_src.TrackModel.Selection.Clear (false);
+                        track_src.TrackModel.Selection.Select (i);
+                        track_list.CenterOn (i);
+                    }
+                }
+            }
         }
     }
 }
