@@ -94,6 +94,7 @@ namespace Banshee.Widgets
             box.ShowAll();
 
             entry.StyleSet += OnInnerEntryStyleSet;
+            entry.StateChanged += OnInnerEntryStateChanged;
             entry.FocusInEvent += OnInnerEntryFocusEvent;
             entry.FocusOutEvent += OnInnerEntryFocusEvent;
             entry.Changed += OnInnerEntryChanged;
@@ -193,15 +194,25 @@ namespace Banshee.Widgets
             return false;
         }
 
-        private void OnInnerEntryStyleSet(object o, StyleSetArgs args)
+        private void UpdateStyle ()
         {
-            Gdk.Color color = entry.Style.Base(StateType.Normal);
-            filter_button.ModifyBg(StateType.Normal, color);
-            clear_button.ModifyBg(StateType.Normal, color);
-
+            Gdk.Color color = entry.Style.Base (entry.State);
+            filter_button.ModifyBg (entry.State, color);
+            clear_button.ModifyBg (entry.State, color);
+            
             box.BorderWidth = (uint)entry.Style.XThickness;
         }
-
+        
+        private void OnInnerEntryStyleSet (object o, StyleSetArgs args)
+        {
+            UpdateStyle ();
+        }
+        
+        private void OnInnerEntryStateChanged (object o, EventArgs args)
+        {
+            UpdateStyle ();
+        }
+        
         private void OnInnerEntryFocusEvent(object o, EventArgs args)
         {
             QueueDraw();
@@ -367,6 +378,7 @@ namespace Banshee.Widgets
             get { return entry.HasFocus; }
             set { entry.HasFocus = true; }
         }
+
         
         public Entry InnerEntry {
             get { return entry; }
@@ -428,7 +440,7 @@ namespace Banshee.Widgets
                 text_gc.Copy(Style.TextGC(StateType.Normal));
                 Gdk.Color color_a = parent.Style.Base(StateType.Normal);
                 Gdk.Color color_b = parent.Style.Text(StateType.Normal);
-                text_gc.RgbFgColor = DrawingUtilities.ColorBlend(color_a, color_b, 0.5);
+                text_gc.RgbFgColor = Hyena.Gui.GtkUtilities.ColorBlend(color_a, color_b);
             }
 
             protected override bool OnExposeEvent(Gdk.EventExpose evnt)
