@@ -54,7 +54,7 @@ namespace Banshee.Lastfm.Radio
     {
         private static string generic_name = Catalog.GetString ("Last.fm Station");
         
-        private LastfmTrackListModel track_model;
+        private MemoryTrackListModel track_model;
         
         private LastfmSource lastfm;
         public LastfmSource LastfmSource {
@@ -92,6 +92,10 @@ namespace Banshee.Lastfm.Radio
 
         private int dbid;
         
+        protected override string TypeUniqueId {
+            get { return Convert.ToString (dbid); }
+        }
+        
         // For StationSources that already exist in the db
         protected StationSource (LastfmSource lastfm, int dbId, string name, string type, string arg, int playCount) : base (generic_name, name, 150)
         {
@@ -102,7 +106,7 @@ namespace Banshee.Lastfm.Radio
             PlayCount = playCount;
             Station = Type.GetStationFor (arg);
 
-            Initialize ();
+            StationInitialize ();
         }
 
         public StationSource (LastfmSource lastfm, string name, string type, string arg) : base (generic_name, name, 150)
@@ -114,17 +118,16 @@ namespace Banshee.Lastfm.Radio
 
             Save ();
 
-            Initialize ();
+            StationInitialize ();
         }
 
-        private void Initialize ()
+        private void StationInitialize ()
         {
-            track_model = new LastfmTrackListModel ();
+            track_model = new MemoryTrackListModel ();
 
             ServiceManager.PlayerEngine.StateChanged += OnPlayerStateChanged;
             lastfm.Connection.StateChanged += HandleConnectionStateChanged;
 
-            Properties.SetString ("ActiveSourceUIResource", "ActiveSourceUI.xml");
             Properties.SetString ("GtkActionPath", "/LastfmStationSourcePopup");
             Properties.SetString ("SourcePropertiesActionLabel", Catalog.GetString ("Edit Last.fm Station"));
             Properties.SetString ("UnmapSourceActionLabel", Catalog.GetString ("Delete Last.fm Station"));
@@ -178,9 +181,6 @@ namespace Banshee.Lastfm.Radio
         public override void Activate ()
         {
             base.Activate ();
-            //shuffle = (action_service.GlobalActions ["ShuffleAction"] as ToggleAction).Active;
-            //(action_service.GlobalActions ["ShuffleAction"] as ToggleAction).Active = false;
-            //Globals.ActionManager["ShuffleAction"].Sensitive = false;
 
             //action_service.GlobalActions ["PreviousAction"].Sensitive = false;
 
@@ -206,9 +206,6 @@ namespace Banshee.Lastfm.Radio
 
         public override void Deactivate ()
         {
-            //(Globals.ActionManager["ShuffleAction"] as ToggleAction).Active = shuffle;
-            //Globals.ActionManager["ShuffleAction"].Sensitive = true;
-
             //Globals.ActionManager["PreviousAction"].Sensitive = true;
         }
 
@@ -235,45 +232,16 @@ namespace Banshee.Lastfm.Radio
             }
         }
 
-        private void SetStatus (string message, bool error)
+        protected override void SetStatus (string message, bool error)
         {
-            SetStatus (message, error, ConnectionState.Connected);
+            base.SetStatus (message, error);
+            LastfmSource.SetStatus (status_message, lastfm, error, ConnectionState.Connected);
         }
 
         private void SetStatus (string message, bool error, ConnectionState state)
         {
-            ThreadAssist.ProxyToMain (delegate {
-                string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
-                Properties.SetString ("Message.Text", String.Format ("{0}", 
-                    String.Format (GLib.Markup.EscapeText (message), status_name)));
-                
-                if (error) {
-                    Properties.SetString ("Message.Icon.Name", "dialog-error");
-                    Properties.SetBoolean ("Message.IsSpinning", false);
-                    
-                    if (state == ConnectionState.NoAccount || state == ConnectionState.InvalidAccount) {
-                        Properties.SetString ("Message.Action.Label", Catalog.GetString ("Account Settings"));
-                        Properties.Set<EventHandler> ("Message.Action.NotifyHandler", delegate {
-                             lastfm.Actions.ShowLoginDialog ();
-                           });
-                    } else {
-                        Properties.RemoveStartingWith ("Message.Action.");
-                      }
-                } else {
-                    Properties.RemoveStartingWith ("Message.Icon.");
-                    Properties.SetBoolean ("Message.IsSpinning", true);
-                    Properties.RemoveStartingWith ("Message.Action.");
-                }
-                
-                Properties.SetBoolean ("Message.CanClose", true);
-            });
-        }
-
-        private void HideStatus ()
-        {
-            ThreadAssist.ProxyToMain (delegate {
-                Properties.RemoveStartingWith ("Message.");
-            });
+            base.SetStatus (message, error);
+            LastfmSource.SetStatus (status_message, lastfm, error, state);
         }
 
         /*public override void ShowPropertiesDialog ()
@@ -428,6 +396,11 @@ namespace Banshee.Lastfm.Radio
             );
         }
 
+        public override bool CanRename {
+            get { return true; }
+        }
+
+
 #region ITrackModelSource Implementation
 
         public TrackListModel TrackModel {
@@ -456,6 +429,10 @@ namespace Banshee.Lastfm.Radio
             throw new Exception ("Should not call DeleteSelectedTracks on StationSource");
         }
 
+        public bool CanAddTracks {
+            get { return false; }
+        }
+
         public bool CanRemoveTracks {
             get { return false; }
         }
@@ -504,14 +481,6 @@ namespace Banshee.Lastfm.Radio
             Save ();
         }
         
-        public override bool CanSearch {
-            get { return false; }
-        }
-
-        public override int Count {
-            get { return 0; }
-        }
-
         public override bool HasProperties {
             get { return true; }
         }
