@@ -364,7 +364,7 @@ namespace Banshee.NotificationArea
                 nf.Urgency = Urgency.Low;
                 nf.Timeout = 4500;
                 if (interface_action_service.PlaybackActions["NextAction"].Sensitive) {
-                    nf.AddAction ("skip-song", Catalog.GetString("Play next item"), OnSongSkipped);
+                    nf.AddAction ("skip-song", Catalog.GetString("Skip this item"), OnSongSkipped);
                 }
                 nf.Show ();
                 
