@@ -54,15 +54,15 @@ namespace Banshee.PlayQueue
         private DatabaseTrackInfo playing_track;
         private bool actions_loaded = false;
         
-        public PlayQueueSource () : base (Catalog.GetString ("Play Queue"), null)
+        public PlayQueueSource () : base (Catalog.GetString ("Play Queue"), null, 0)
         {
             BindToDatabase ();
             
-            Order = 0;
-            Properties.SetString ("Icon.Name", "audio-x-generic");
+            Order = 10;
+            Properties.SetString ("Icon.Name", "source-playlist");
             Properties.SetString ("RemoveTracksActionLabel", Catalog.GetString ("Remove From Play Queue"));
             
-            ((TrackListDatabaseModel)TrackModel).ForcedSortQuery = "CorePlaylistEntries.EntryID ASC";
+            ((DatabaseTrackListModel)TrackModel).ForcedSortQuery = "CorePlaylistEntries.EntryID ASC";
             
             ServiceManager.PlayerEngine.EventChanged += OnPlayerEngineEventChanged;
             ServiceManager.PlaybackController.Transition += OnCanonicalPlaybackControllerTransition;
@@ -101,10 +101,22 @@ namespace Banshee.PlayQueue
             UpdateActions ();
             ServiceManager.SourceManager.ActiveSourceChanged += delegate { UpdateActions (); };
 
-            ServiceManager.SourceManager.Library.TracksChanged += HandleTracksChanged;
-            ServiceManager.SourceManager.Library.TracksDeleted += HandleTracksDeleted;
+            // TODO listen to all primary sources, and handle transient primary sources
+            ServiceManager.SourceManager.MusicLibrary.TracksChanged += HandleTracksChanged;
+            ServiceManager.SourceManager.MusicLibrary.TracksDeleted += HandleTracksDeleted;
+            ServiceManager.SourceManager.VideoLibrary.TracksChanged += HandleTracksChanged;
+            ServiceManager.SourceManager.VideoLibrary.TracksDeleted += HandleTracksDeleted;
+
+            SetAsPlayingSource ();
         }
         
+        private void SetAsPlayingSource ()
+        {
+            if (Count > 0 && ServiceManager.PlayerEngine.CurrentState != PlayerEngineState.Playing) {
+                ServiceManager.PlaybackController.Source = this;
+            }
+        }
+
         public void Dispose ()
         {
             if (ClearOnQuitSchema.Get ()) {
@@ -127,6 +139,12 @@ namespace Banshee.PlayQueue
                 ", special_playlist_name));
             }
         }
+
+        protected override void OnTracksAdded ()
+        {
+            base.OnTracksAdded ();
+            SetAsPlayingSource ();
+        }
         
         protected override void OnUpdated ()
         {
@@ -155,12 +173,12 @@ namespace Banshee.PlayQueue
         
         private void OnAddToPlayQueue (object o, EventArgs args)
         {
-            AddSelectedTracks (ServiceManager.Get<InterfaceActionService> ().TrackActions.TrackSelector.TrackModel);
+            AddSelectedTracks (ServiceManager.SourceManager.ActiveSource);
         }
         
         private void OnClearPlayQueue (object o, EventArgs args)
         {
-            RemoveTrackRange ((TrackListDatabaseModel)TrackModel, new Hyena.Collections.RangeCollection.Range (0, Count));
+            RemoveTrackRange ((DatabaseTrackListModel)TrackModel, new Hyena.Collections.RangeCollection.Range (0, Count));
             Reload ();
         }
         
@@ -191,10 +209,10 @@ namespace Banshee.PlayQueue
         
         void IBasicPlaybackController.First ()
         {
-            ((IBasicPlaybackController)this).Next ();
+            ((IBasicPlaybackController)this).Next (false);
         }
         
-        void IBasicPlaybackController.Next ()
+        void IBasicPlaybackController.Next (bool restart)
         {
             RemovePlayingTrack ();
             
@@ -208,7 +226,7 @@ namespace Banshee.PlayQueue
             ServiceManager.PlayerEngine.OpenPlay ((DatabaseTrackInfo)TrackModel[0]);
         }
         
-        void IBasicPlaybackController.Previous ()
+        void IBasicPlaybackController.Previous (bool restart)
         {
         }
         
