@@ -37,14 +37,21 @@ namespace Hyena.Data.Gui
 {
     public class ColumnCellText : ColumnCell
     {
+        private static bool use_cairo_pango;
+        
+        static ColumnCellText ()
+        {
+            use_cairo_pango = String.IsNullOrEmpty (Environment.GetEnvironmentVariable ("USE_GTK_PANGO"));
+            Log.DebugFormat ("Text renderer: {0}", use_cairo_pango ? "Cairo" : "GTK");
+        }
+        
         public delegate string DataHandler ();
     
         private Pango.Weight font_weight = Pango.Weight.Normal;
         private Pango.EllipsizeMode ellipsize_mode = Pango.EllipsizeMode.End;
         private int text_width;
         private int text_height;
-        private bool use_cairo_pango = !String.IsNullOrEmpty (Environment.GetEnvironmentVariable ("USE_CAIRO_PANGO"));
-
+        
         public ColumnCellText (string property, bool expand) : base (property, expand)
         {
         }
@@ -52,19 +59,17 @@ namespace Hyena.Data.Gui
         public override void Render (CellContext context, StateType state, double cellWidth, double cellHeight)
         {
             context.Layout.Width = (int)((cellWidth - 8) * Pango.Scale.PangoScale);
-            context.Layout.FontDescription = context.Widget.PangoContext.FontDescription.Copy ();
             context.Layout.FontDescription.Weight = font_weight;
-            context.Layout.Ellipsize = ellipsize_mode;
+            context.Layout.Ellipsize = EllipsizeMode;
             
             context.Layout.SetText (Text);
             context.Layout.GetPixelSize (out text_width, out text_height);
             
             if (use_cairo_pango) {
                 context.Context.MoveTo (4, ((int)cellHeight - text_height) / 2);
-                PangoCairoHelper.LayoutPath (context.Context, context.Layout);
                 context.Context.Color = context.Theme.Colors.GetWidgetColor (
                     context.TextAsForeground ? GtkColorClass.Foreground : GtkColorClass.Text, state);
-                context.Context.Fill ();
+                PangoCairoHelper.ShowLayout (context.Context, context.Layout);
             } else {
                 Style.PaintLayout (context.Widget.Style, context.Drawable, state, !context.TextAsForeground, 
                     context.Clip, context.Widget, "text", context.Area.X + 4, 
@@ -73,7 +78,7 @@ namespace Hyena.Data.Gui
         }
         
         protected virtual string Text {
-            get { return BoundObject == null ? String.Empty : BoundObject.ToString(); }
+            get { return BoundObject == null ? String.Empty : BoundObject.ToString (); }
         }
         
         protected int TextWidth {
@@ -84,10 +89,6 @@ namespace Hyena.Data.Gui
             get { return text_height; }
         }
         
-        internal bool UseCairoPango {
-            set { use_cairo_pango = value; }
-        }
-        
         public virtual Pango.Weight FontWeight {
             get { return font_weight; }
             set { font_weight = value; }
@@ -97,5 +98,15 @@ namespace Hyena.Data.Gui
             get { return ellipsize_mode; }
             set { ellipsize_mode = value; }
         }
+        
+        internal static int ComputeRowHeight (Widget widget)
+        {
+            int w_width, row_height;
+            Pango.Layout layout = new Pango.Layout (widget.PangoContext);
+            layout.SetText ("W");
+            layout.GetPixelSize (out w_width, out row_height);
+            layout.Dispose ();
+            return row_height + 8;
+        }
     }
 }
