@@ -35,9 +35,8 @@ using Selection = Hyena.Collections.Selection;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Container
+    public partial class ListView<T> : Widget
     {
-        private int last_click_row_index = -1;
         private int focused_row_index = -1;
 
         private Adjustment vadjustment;
@@ -45,6 +44,11 @@ namespace Hyena.Data.Gui
             get { return vadjustment; }
         }
         
+        private Adjustment hadjustment;
+        public Adjustment Hadjustment {
+            get { return hadjustment; }
+        }
+        
         private SelectionProxy selection_proxy = new SelectionProxy ();
         public SelectionProxy SelectionProxy {
             get { return selection_proxy; }
@@ -109,7 +113,7 @@ namespace Hyena.Data.Gui
             }
 
             focused_row_index = row_index;
-            InvalidateListWindow ();
+            InvalidateList ();
             return true;
         }
         
@@ -192,54 +196,68 @@ namespace Hyena.Data.Gui
             
             return base.OnKeyPressEvent (press);
         }
+        
+#region OnButtonPress
 
-        protected override bool OnButtonPressEvent (Gdk.EventButton press)
+        protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
         {
             HasFocus = true;
+            if (header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
+                return OnHeaderButtonPressEvent (evnt);
+            } else if (list_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && model != null) {
+                return OnListButtonPressEvent (evnt);
+            }
+            return true;
+        }
+        
+        private bool OnHeaderButtonPressEvent (Gdk.EventButton evnt)
+        {
+            int x = (int)evnt.X - header_interaction_alloc.X;
+            int y = (int)evnt.Y - header_interaction_alloc.Y;
             
-            if (press.Window == header_window) {
-                if (press.Button == 3 && ColumnController.EnableColumnMenu) {
-                    Column menu_column = GetColumnAt ((int)press.X);
-                    if (menu_column != null) {
-                        OnColumnRightClicked (menu_column, (int)press.X, (int)press.Y);
-                    }
-                    return true;
-                } else if (press.Button != 1) {
-                    return true;
+            if (evnt.Button == 3 && ColumnController.EnableColumnMenu) {
+                Column menu_column = GetColumnAt (x);
+                if (menu_column != null) {
+                    OnColumnRightClicked (menu_column, x + Allocation.X, y + Allocation.Y);
                 }
-                
-                Gtk.Drag.SourceUnset (this);
-                
-                Column column = GetColumnForResizeHandle ((int)press.X);
+                return true;
+            } else if (evnt.Button != 1) {
+                return true;
+            }
+            
+            Gtk.Drag.SourceUnset (this);
+            
+            Column column = GetColumnForResizeHandle (x);
+            if (column != null) {
+                resizing_column_index = GetCachedColumnForColumn (column).Index;
+            } else {
+                column = GetColumnAt (x);
                 if (column != null) {
-                    resizing_column_index = GetCachedColumnForColumn (column).Index;
-                } else {
-                    column = GetColumnAt ((int)press.X);
-                    if (column != null) {
-                        CachedColumn column_c = GetCachedColumnForColumn (column);
-                        pressed_column_index = column_c.Index;
-                        pressed_column_x_start = (int)press.X;
-                        pressed_column_x_offset = pressed_column_x_start - column_c.X1;
-                    }
+                    CachedColumn column_c = GetCachedColumnForColumn (column);
+                    pressed_column_index = column_c.Index;
+                    pressed_column_x_start = x;
+                    pressed_column_x_offset = pressed_column_x_start - column_c.X1;
+                    pressed_column_x_start_hadjustment = (int)hadjustment.Value;
                 }
-                
-                return true;
             }
             
-            if (press.Window != list_window || model == null) {
-                return true;
-            }
+            return true;
+        }
+        
+        private bool OnListButtonPressEvent (Gdk.EventButton evnt)
+        {
+            int y = (int)evnt.Y - list_interaction_alloc.Y;
             
             GrabFocus ();
             
-            int row_index = GetRowAtY ((int)press.Y);
+            int row_index = GetRowAtY (y);
 
             if (row_index >= Model.Count) {
                 return true;
             }
             
-            if (press.Button == 1 && press.Type != Gdk.EventType.TwoButtonPress && 
-                (press.State & Gdk.ModifierType.ControlMask) == 0 && Selection.Contains (row_index)) {
+            if (evnt.Button == 1 && evnt.Type != Gdk.EventType.TwoButtonPress && 
+                (evnt.State & Gdk.ModifierType.ControlMask) == 0 && Selection.Contains (row_index)) {
                 return true;
             }
 
@@ -248,23 +266,21 @@ namespace Hyena.Data.Gui
                 return true;
             }
 
-            if (press.Button == 1 && press.Type == Gdk.EventType.TwoButtonPress && 
-                row_index == last_click_row_index) {
+            if (evnt.Button == 1 && evnt.Type == Gdk.EventType.TwoButtonPress) {
                 OnRowActivated ();
-                last_click_row_index = -1;
             } else {
-                if ((press.State & Gdk.ModifierType.ControlMask) != 0) {
-                    if (press.Button == 3) {
+                if ((evnt.State & Gdk.ModifierType.ControlMask) != 0) {
+                    if (evnt.Button == 3) {
                         if (!Selection.Contains (row_index)) {
                             Selection.Select (row_index);
                         }
                     } else {
                         Selection.ToggleSelect (row_index);
                     }
-                } else if ((press.State & Gdk.ModifierType.ShiftMask) != 0) {
+                } else if ((evnt.State & Gdk.ModifierType.ShiftMask) != 0) {
                     Selection.SelectFromFirst (row_index, true);
                 } else {
-                    if (press.Button == 3) {
+                    if (evnt.Button == 3) {
                         if (!Selection.Contains (row_index)) {
                             Selection.Clear (false);
                             Selection.Select (row_index);
@@ -277,130 +293,175 @@ namespace Hyena.Data.Gui
 
                 FocusRow (row_index);
 
-                if (press.Button == 3) {
-                    last_click_row_index = -1;
+                if (evnt.Button == 3) {
                     OnPopupMenu ();
-                } else {
-                    last_click_row_index = row_index;
                 }
             }
             
-            InvalidateListWindow ();
-            
+            InvalidateList ();
             return true;
         }
         
+#endregion
+
+#region OnButtonRelease
+        
         protected override bool OnButtonReleaseEvent (Gdk.EventButton evnt)
         {
-            if (evnt.Window == header_window) {
-                OnDragSourceSet ();
-                
-                if (resizing_column_index >= 0) {
-                    pressed_column_index = -1;
-                    resizing_column_index = -1;
-                    header_window.Cursor = null;
-                    return true;
-                }
-                
-                if (pressed_column_index >= 0 && pressed_column_is_dragging) {
-                    pressed_column_is_dragging = false;
-                    pressed_column_index = -1;
-                    header_window.Cursor = null;
-                    InvalidateHeaderWindow ();
-                    InvalidateListWindow ();
-                    return true;
-                }
+            OnDragSourceSet ();
+            StopDragScroll ();
             
-                Column column = column_cache[pressed_column_index].Column;
-                if (column != null && Model is ISortable && column is ISortableColumn) {
-                    ((ISortable)Model).Sort ((ISortableColumn)column);
-                    Model.Reload ();
-                    InvalidateHeaderWindow ();
-                }
-                
+            if (resizing_column_index >= 0) {
                 pressed_column_index = -1;
-            } else if (evnt.Window == list_window && model != null &&
+                resizing_column_index = -1;
+                GdkWindow.Cursor = null;
+                return true;
+            }
+            
+            if (pressed_column_index >= 0 && pressed_column_is_dragging) {
+                pressed_column_is_dragging = false;
+                pressed_column_index = -1;
+                GdkWindow.Cursor = null;
+                InvalidateHeader ();
+                InvalidateList ();
+                return true;
+            }
+            
+            if (header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
+                return OnHeaderButtonRelease (evnt);
+            } else if (list_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && model != null &&
                 (evnt.State & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0) {
-                GrabFocus ();
-                
-                int row_index = GetRowAtY ((int)evnt.Y);
-
-                if (row_index >= Model.Count) {
-                    return true;
-                }
-
-                object item = model[row_index];
-                if (item == null) {
-                    return true;
-                }
-                
-                if (Selection.Contains (row_index) && Selection.Count > 1) {
-                    Selection.Clear (false);
-                    Selection.Select (row_index);
-                    FocusRow (row_index);
-                    return true;
-                }
+                return OnListButtonRelease (evnt);
             }
 
             return true;
         }
         
+        private bool OnHeaderButtonRelease (Gdk.EventButton evnt)
+        {
+            Column column = column_cache[pressed_column_index].Column;
+            if (column != null && Model is ISortable && column is ISortableColumn) {
+                ((ISortable)Model).Sort ((ISortableColumn)column);
+                Model.Reload ();
+                RecalculateColumnSizes ();
+                RegenerateColumnCache ();
+                InvalidateHeader ();
+            }
+            
+            pressed_column_index = -1;
+            return true;
+        }
+        
+        private bool OnListButtonRelease (Gdk.EventButton evnt)
+        {
+            int y = (int)evnt.Y - list_interaction_alloc.Y;
+            
+            GrabFocus ();
+            
+            int row_index = GetRowAtY (y);
+
+            if (row_index >= Model.Count) {
+                return true;
+            }
+
+            object item = model[row_index];
+            if (item == null) {
+                return true;
+            }
+            
+            if (Selection.Contains (row_index) && Selection.Count > 1) {
+                Selection.Clear (false);
+                Selection.Select (row_index);
+                FocusRow (row_index);
+            }
+            return true;
+        }
+        
+#endregion
+        
         protected override bool OnMotionNotifyEvent (Gdk.EventMotion evnt)
         {
-            if (evnt.Window == header_window) {
-                if (pressed_column_index >= 0 && !pressed_column_is_dragging && 
-                    Gtk.Drag.CheckThreshold (this, pressed_column_x_start, 0, (int)evnt.X, 0)) {
-                    pressed_column_is_dragging = true;
-                    InvalidateHeaderWindow ();
-                    InvalidateListWindow ();
-                }
-                
-                if (pressed_column_is_dragging) {
-                    header_window.Cursor = drag_cursor;
-                    
-                    Column swap_column = GetColumnAt ((int)evnt.X);
-                    
-                    if (swap_column != null) {
-                        CachedColumn swap_column_c = GetCachedColumnForColumn (swap_column);
-                        bool reorder = false;
-                        
-                        if (swap_column_c.Index < pressed_column_index) {
-                            // Moving from right to left
-                            reorder = pressed_column_x_drag <= swap_column_c.X1 + swap_column_c.Width / 2;
-                        } else if (swap_column_c.Index > pressed_column_index) {
-                            // Moving from left to right
-                            reorder = pressed_column_x_drag + column_cache[pressed_column_index].Width >= 
-                                swap_column_c.X1 + swap_column_c.Width / 2;
-                        }
-                        
-                        if (reorder) {
-                            int actual_pressed_index = ColumnController.IndexOf (column_cache[pressed_column_index].Column);
-                            int actual_swap_index = ColumnController.IndexOf (swap_column_c.Column);
-                            ColumnController.Reorder (actual_pressed_index, actual_swap_index);
-                            pressed_column_index = swap_column_c.Index;
-                            RegenerateColumnCache ();
-                        }
-                    }
-                    
-                    pressed_column_x_drag = (int)evnt.X - pressed_column_x_offset;
-                    
-                    InvalidateHeaderWindow ();
-                    InvalidateListWindow ();
-                    return true;
-                }
+            int x = (int)evnt.X - header_interaction_alloc.X;
             
-                header_window.Cursor = resizing_column_index >= 0 || GetColumnForResizeHandle ((int)evnt.X) != null 
-                    ? resize_x_cursor 
-                    : null;
-                
-                if (resizing_column_index >= 0) {
-                    ResizeColumn (evnt.X);
-                }
+            if (pressed_column_index >= 0 && !pressed_column_is_dragging && 
+                Gtk.Drag.CheckThreshold (this, pressed_column_x_start, 0, x, 0)) {
+                pressed_column_is_dragging = true;
+                InvalidateHeader ();
+                InvalidateList ();
+            }
+            
+            pressed_column_x = x;
+            
+            if (OnMotionNotifyEvent (x)) {
+                return true;
+            }
+            
+            GdkWindow.Cursor = (resizing_column_index >= 0 || GetColumnForResizeHandle (x) != null) &&
+                header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) 
+                ? resize_x_cursor 
+                : null;
+            
+            if (resizing_column_index >= 0) {
+                ResizeColumn (x);
             }
             
             return true;
         }
         
+        private bool OnMotionNotifyEvent (int x)
+        {
+            if (!pressed_column_is_dragging) {
+                return false;
+            }
+            
+            OnDragScroll (OnDragHScrollTimeout, header_interaction_alloc.Width * 0.1, header_interaction_alloc.Width, x);
+            
+            GdkWindow.Cursor = drag_cursor;
+            
+            Column swap_column = GetColumnAt (x);
+            
+            if (swap_column != null) {
+                CachedColumn swap_column_c = GetCachedColumnForColumn (swap_column);
+                bool reorder = false;
+                
+                if (swap_column_c.Index < pressed_column_index) {
+                    // Moving from right to left
+                    reorder = pressed_column_x_drag <= swap_column_c.X1 + swap_column_c.Width / 2;
+                } else if (swap_column_c.Index > pressed_column_index) {
+                    // Moving from left to right
+                    reorder = pressed_column_x_drag + column_cache[pressed_column_index].Width >= 
+                        swap_column_c.X1 + swap_column_c.Width / 2;
+                }
+                
+                if (reorder) {
+                    int actual_pressed_index = ColumnController.IndexOf (column_cache[pressed_column_index].Column);
+                    int actual_swap_index = ColumnController.IndexOf (swap_column_c.Column);
+                    ColumnController.Reorder (actual_pressed_index, actual_swap_index);
+                    pressed_column_index = swap_column_c.Index;
+                    RegenerateColumnCache ();
+                }
+            }
+            
+            pressed_column_x_drag = x - pressed_column_x_offset - (pressed_column_x_start_hadjustment - (int)hadjustment.Value);
+            
+            QueueDraw ();
+            return true;
+        }
+        
+        private bool OnDragHScrollTimeout ()
+        {
+            ScrollTo (hadjustment, hadjustment.Value + (drag_scroll_velocity * drag_scroll_velocity_max));
+            OnMotionNotifyEvent (pressed_column_x);
+            return true;
+        }
+        
+        protected override bool OnLeaveNotifyEvent (Gdk.EventCrossing evnt)
+        {
+            GdkWindow.Cursor = null;
+            return base.OnLeaveNotifyEvent (evnt);
+        }
+
+        
         protected override bool OnFocusInEvent (Gdk.EventFocus evnt)
         {
             return base.OnFocusInEvent (evnt);
@@ -445,43 +506,80 @@ namespace Hyena.Data.Gui
 
 #region Adjustments & Scrolling
         
+        private void UpdateAdjustments ()
+        {
+            UpdateAdjustments (null, null);
+        }
+        
         private void UpdateAdjustments (Adjustment hadj, Adjustment vadj)
         {
+            if (hadj != null) {
+                hadjustment = hadj;
+            }
+            
             if (vadj != null) {
                 vadjustment = vadj;
             }
             
+            if (hadjustment != null) {
+                hadjustment.Upper = header_width;
+                hadjustment.StepIncrement = 10.0;
+                if (hadjustment.Value + hadjustment.PageSize > hadjustment.Upper) {
+                    hadjustment.Value = hadjustment.Upper - hadjustment.PageSize;
+                }
+            }
+            
             if (vadjustment != null && model != null) {
                 vadjustment.Upper = (RowHeight * (model.Count));
                 vadjustment.StepIncrement = RowHeight;
+                if (vadjustment.Value + vadjustment.PageSize > vadjustment.Upper) {
+                    vadjustment.Value = vadjustment.Upper - vadjustment.PageSize;
+                }
             }
             
+            hadjustment.Change ();
             vadjustment.Change ();
         }
         
-        private void OnAdjustmentChanged (object o, EventArgs args)
+        private void OnHadjustmentChanged (object o, EventArgs args)
         {
-            InvalidateListWindow ();
+            InvalidateHeader ();
+            InvalidateList (false);
+        }
+        
+        private void OnVadjustmentChanged (object o, EventArgs args)
+        {
+            InvalidateList (false);
         }
         
         public void ScrollTo (double val)
         {
-            vadjustment.Value = Math.Max (0.0, Math.Min (val, vadjustment.Upper - vadjustment.PageSize));
+            ScrollTo (vadjustment, val);
+        }
+        
+        private void ScrollTo (Adjustment adjustment, double val)
+        {
+            adjustment.Value = Math.Max (0.0, Math.Min (val, adjustment.Upper - adjustment.PageSize));
         }
 
-        public void ScrollToRow (int row_index)
+        public void ScrollTo (int index)
         {
-            ScrollTo (GetYAtRow (row_index));
+            ScrollTo (GetYAtRow (index));
+        }
+
+        public void CenterOn (int index)
+        {
+            ScrollTo (index - RowsInView/2 + 1);
         }
                 
         protected override void OnSetScrollAdjustments (Adjustment hadj, Adjustment vadj)
         {
-            if (vadj == null || hadj == null) {
+            if (hadj == null || vadj == null) {
                 return;
             }
             
-            vadj.ValueChanged += OnAdjustmentChanged;
-            hadj.ValueChanged += OnAdjustmentChanged;
+            hadj.ValueChanged += OnHadjustmentChanged;
+            vadj.ValueChanged += OnVadjustmentChanged;
             
             UpdateAdjustments (hadj, vadj);
         }
