@@ -38,7 +38,9 @@ using GtkColorClass=Hyena.Gui.Theming.GtkColorClass;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Container
+    public delegate int ListViewRowHeightHandler (Widget widget);
+
+    public partial class ListView<T> : Widget
     {
         // The list is rendered to an off-screen Drawable; the "canvas". The canvas is large enough to hold
         // the maximum number of rows that can be seen in the list. When new rows comes into view b/c of
@@ -56,14 +58,23 @@ namespace Hyena.Data.Gui
         private bool render_everything;
         
         private Cairo.Context cairo_context;
-        
-        private Pango.Layout header_pango_layout;
-        private Pango.Layout list_pango_layout;
+        private CellContext cell_context;
+        private Pango.Layout pango_layout;
         
         private Theme theme;
         protected Theme Theme {
             get { return theme; }
         }
+        
+        protected override void OnStyleSet (Style old_style)
+        {
+            base.OnStyleSet (old_style);
+            RecomputeRowHeight = true;
+            theme = new GtkTheme (this);
+            cell_context = new CellContext ();
+            cell_context.Theme = theme;
+            cell_context.Widget = this;
+        }
          
         protected override bool OnExposeEvent (EventExpose evnt)
         {
@@ -81,6 +92,7 @@ namespace Hyena.Data.Gui
             if (header_visible && column_controller != null) {
                 PaintHeader (damage);
             }
+            
             Theme.DrawFrameBorder (cairo_context, Allocation);
             
             // Then we render the list to the offscreen canvas.
@@ -88,11 +100,14 @@ namespace Hyena.Data.Gui
                 canvas1 = new Pixmap (GdkWindow, canvas_alloc.Width, canvas_alloc.Height);
                 render_everything = true;
             }
-            PaintList (damage);
+            
+            PaintList ();
             
             // Now we blit the offscreen canvas onto the GdkWindow.
-            GdkWindow.DrawDrawable (Style.BaseGC (StateType.Normal), canvas1, 0, (int)vadjustment.Value % RowHeight,
-                list_rendering_alloc.X, list_rendering_alloc.Y, list_rendering_alloc.Width, list_rendering_alloc.Height);
+            GdkWindow.DrawDrawable (Style.BaseGC (StateType.Normal), canvas1,
+                (int)hadjustment.Value, (int)vadjustment.Value % RowHeight,
+                list_rendering_alloc.X, list_rendering_alloc.Y,
+                list_rendering_alloc.Width, list_rendering_alloc.Height);
             
             // Finally, render the dragging box and dispose of the cairo context.
             PaintDraggingColumn (damage);
@@ -112,25 +127,30 @@ namespace Hyena.Data.Gui
             cairo_context.Rectangle (clip.X, clip.Y, clip.Width, clip.Height);
             cairo_context.Clip ();
             
-            header_pango_layout = PangoCairoHelper.CreateLayout (cairo_context);
+            CairoExtensions.CreateLayout (this, cairo_context, ref pango_layout);
             Theme.DrawHeaderBackground (cairo_context, header_rendering_alloc);
             
             Rectangle cell_area = new Rectangle ();
             cell_area.Y = header_rendering_alloc.Y;
             cell_area.Height = header_rendering_alloc.Height;
+            
+            cell_context.Context = cairo_context;
+            cell_context.Drawable = GdkWindow;
+            cell_context.Layout = pango_layout;
+            cell_context.TextAsForeground = true;
 
             for (int ci = 0; ci < column_cache.Length; ci++) {
                 if (pressed_column_is_dragging && pressed_column_index == ci) {
                     continue;
                 }
                 
-                cell_area.X = column_cache[ci].X1 + Theme.TotalBorderWidth + header_rendering_alloc.X;
+                cell_area.X = column_cache[ci].X1 + Theme.TotalBorderWidth + header_rendering_alloc.X - (int)hadjustment.Value;
                 cell_area.Width = column_cache[ci].Width;
                 PaintHeaderCell (cell_area, clip, ci, false);
             }
             
             if (pressed_column_is_dragging && pressed_column_index >= 0) {
-                cell_area.X = pressed_column_x_drag + Allocation.X;
+                cell_area.X = pressed_column_x_drag + Allocation.X - (int)hadjustment.Value;
                 cell_area.Width = column_cache[pressed_column_index].Width;
                 PaintHeaderCell (cell_area, clip, pressed_column_index, true);
             }
@@ -161,8 +181,9 @@ namespace Hyena.Data.Gui
             if (cell != null) {
                 cairo_context.Save ();
                 cairo_context.Translate (area.X, area.Y);
-                cell.Render (new CellContext (cairo_context, header_pango_layout, this, GdkWindow, 
-                    theme, area, clip), StateType.Normal, area.Width, area.Height);
+                cell_context.Area = area;
+                cell_context.Clip = clip;
+                cell.Render (cell_context, StateType.Normal, area.Width, area.Height);
                 cairo_context.Restore ();
             }
             
@@ -171,7 +192,7 @@ namespace Hyena.Data.Gui
             }
         }
 
-        private void PaintList (Rectangle clip)
+        private void PaintList ()
         {
             if (model == null) {
                 return;
@@ -180,7 +201,9 @@ namespace Hyena.Data.Gui
             // Render the sort effect to the GdkWindow.
             if (sort_column_index != -1 && (!pressed_column_is_dragging || pressed_column_index != sort_column_index)) {
                 CachedColumn col = column_cache[sort_column_index];
-                Theme.DrawRowRule (cairo_context, list_rendering_alloc.X + col.X1, header_rendering_alloc.Bottom + Theme.BorderWidth,
+                Theme.DrawRowRule (cairo_context,
+                    list_rendering_alloc.X + col.X1 - (int)hadjustment.Value,
+                    header_rendering_alloc.Bottom + Theme.BorderWidth,
                     col.Width, list_rendering_alloc.Height + Theme.InnerBorderWidth * 2);
             }
 
@@ -206,7 +229,7 @@ namespace Hyena.Data.Gui
             // Build a cairo context for the primary canvas.
             Cairo.Context tmp_cr = cairo_context;
             cairo_context = CairoHelper.Create (canvas1);
-            list_pango_layout = PangoCairoHelper.CreateLayout (cairo_context);
+            CairoExtensions.CreateLayout (this, cairo_context, ref pango_layout);
             
             // Render the background to the primary canvas.
             Theme.DrawListBackground (cairo_context, canvas_alloc, true);
@@ -231,7 +254,7 @@ namespace Hyena.Data.Gui
                 // render the new stuff in situ at the bottom.
                 int delta = (last_row - canvas_last_row) * RowHeight;
                 canvas1.DrawDrawable (Style.BaseGC (StateType.Normal), canvas2, 0, delta, 0, 0,
-                    list_rendering_alloc.Width, canvas_alloc.Height - delta);
+                    canvas_alloc.Width, canvas_alloc.Height - delta);
                 
                 // If the bottom of the stuff we're shifting up is part of a selection
                 // that continues down into the new stuff, be sure that we render the
@@ -247,7 +270,7 @@ namespace Hyena.Data.Gui
                 // render the new stuff in situ at the top.
                 int delta = (canvas_first_row - first_row) * RowHeight;
                 canvas1.DrawDrawable (Style.BaseGC (StateType.Normal), canvas2, 0, 0, 0, delta,
-                    list_rendering_alloc.Width, canvas_alloc.Height - delta);
+                    canvas_alloc.Width, canvas_alloc.Height - delta);
                 
                 // If the top of the stuff we're shifting down is part of a selection
                 // that continues up into the new stuff, be sure that we render the
@@ -259,7 +282,7 @@ namespace Hyena.Data.Gui
                 last_row = canvas_first_row;
             }
             
-            PaintRows (first_row, Math.Min (model.Count, last_row), first_row_y, clip);
+            PaintRows (first_row, Math.Min (model.Count, last_row), first_row_y);
             
             // Destroy the cairo context.
             ((IDisposable)cairo_context.Target).Dispose ();
@@ -270,8 +293,14 @@ namespace Hyena.Data.Gui
             canvas_last_row = bottom;
         }
         
-        private void PaintRows (int first_row, int last_row, int first_row_y, Rectangle clip)
+        private void PaintRows (int first_row, int last_row, int first_row_y)
         {
+            cell_context.Context = cairo_context;
+            cell_context.Drawable = canvas1;
+            cell_context.Layout = pango_layout;
+            cell_context.Clip = canvas_alloc;
+            cell_context.TextAsForeground = false;
+            
             Rectangle selected_focus_alloc = Rectangle.Zero;
             Rectangle single_list_alloc = new Rectangle ();
             
@@ -316,11 +345,11 @@ namespace Hyena.Data.Gui
                         CairoCorners corners = CairoCorners.All;
                         
                         if (Selection.Contains (ri - 1)) {
-                            corners ^= CairoCorners.TopLeft | CairoCorners.TopRight;
+                            corners &= ~(CairoCorners.TopLeft | CairoCorners.TopRight);
                         }
                         
                         if (Selection.Contains (ri + 1)) {
-                            corners ^= CairoCorners.BottomLeft | CairoCorners.BottomRight;
+                            corners &= ~(CairoCorners.BottomLeft | CairoCorners.BottomRight);
                         }
                         
                         Theme.DrawRowSelection (cairo_context, single_list_alloc.X, single_list_alloc.Y, 
@@ -329,12 +358,11 @@ namespace Hyena.Data.Gui
                     }
                     
                     if (selection_height > 0) {
-                        Theme.DrawRowSelection (
-                            cairo_context, 0, selection_y, canvas_alloc.Width, selection_height);
+                        Theme.DrawRowSelection (cairo_context, 0, selection_y, canvas_alloc.Width, selection_height);
                         selection_height = 0;
                     }
                     
-                    PaintRow (ri, clip, single_list_alloc, StateType.Normal);
+                    PaintRow (ri, single_list_alloc, StateType.Normal);
                 }
                 
                 single_list_alloc.Y += single_list_alloc.Height;
@@ -353,11 +381,11 @@ namespace Hyena.Data.Gui
             
             foreach (int ri in selected_rows) {
                 single_list_alloc.Y = (ri - first_row) * single_list_alloc.Height + first_row_y;
-                PaintRow (ri, clip, single_list_alloc, StateType.Selected);
+                PaintRow (ri, single_list_alloc, StateType.Selected);
             }
         }
 
-        private void PaintRow (int row_index, Rectangle clip, Rectangle area, StateType state)
+        private void PaintRow (int row_index, Rectangle area, StateType state)
         {
             if (column_cache == null) {
                 return;
@@ -377,18 +405,18 @@ namespace Hyena.Data.Gui
                 
                 cell_area.Width = column_cache[ci].Width;
                 cell_area.X = column_cache[ci].X1 + area.X;
-                PaintCell (item, ci, row_index, cell_area, clip, sensitive ? state : StateType.Insensitive, false);
+                PaintCell (item, ci, row_index, cell_area, sensitive ? state : StateType.Insensitive, false);
             }
             
             if (pressed_column_is_dragging && pressed_column_index >= 0) {
                 cell_area.Width = column_cache[pressed_column_index].Width;
                 cell_area.X = pressed_column_x_drag - list_interaction_alloc.X;
-                PaintCell (item, pressed_column_index, row_index, cell_area, clip, state, true);
+                PaintCell (item, pressed_column_index, row_index, cell_area, state, true);
             }
         }
         
         private void PaintCell (object item, int column_index, int row_index, Rectangle area, 
-            Rectangle clip, StateType state, bool dragging)
+            StateType state, bool dragging)
         {
             ColumnCell cell = column_cache[column_index].Column.GetCell (0);
             cell.BindListItem (item);
@@ -403,8 +431,8 @@ namespace Hyena.Data.Gui
             
             cairo_context.Save ();
             cairo_context.Translate (area.X, area.Y);
-            cell.Render (new CellContext (cairo_context, list_pango_layout, this, canvas1, theme, area, clip), 
-                dragging ? StateType.Normal : state, area.Width, area.Height);
+            cell_context.Area = area;
+            cell.Render (cell_context, dragging ? StateType.Normal : state, area.Width, area.Height);
             cairo_context.Restore ();
         }
         
@@ -416,7 +444,7 @@ namespace Hyena.Data.Gui
             
             CachedColumn column = column_cache[pressed_column_index];
             
-            int x = pressed_column_x_drag + Allocation.X + 1;
+            int x = pressed_column_x_drag + Allocation.X + 1 - (int)hadjustment.Value;
             
             Cairo.Color fill_color = Theme.Colors.GetWidgetColor (GtkColorClass.Base, StateType.Normal);
             fill_color.A = 0.45;
@@ -477,21 +505,44 @@ namespace Hyena.Data.Gui
             }
         }
         
-        private int row_height = 0;
-        protected int RowHeight {
+        private ListViewRowHeightHandler row_height_handler;
+        public virtual ListViewRowHeightHandler RowHeightProvider {
+            get { return row_height_handler; }
+            set {
+                if (value != row_height_handler) {
+                    row_height_handler = value;
+                    RecomputeRowHeight = true;
+                }
+            }
+        }
+        
+        private bool recompute_row_height = true;
+        protected bool RecomputeRowHeight {
+            get { return recompute_row_height; }
+            set { 
+                recompute_row_height = value;
+                if (value && IsMapped && IsRealized) {
+                    QueueDraw ();
+                }
+            }
+        }
+        
+        private int row_height = 32;
+        private int RowHeight {
             get {
-                if (row_height == 0) {
-                    int w_width;
-                    Pango.Layout layout = new Pango.Layout (PangoContext);
-                    layout.SetText ("W");
-                    layout.GetPixelSize (out w_width, out row_height);
-                    row_height += 8;
+                if (RecomputeRowHeight) {
+                    row_height = RowHeightProvider != null 
+                        ? RowHeightProvider (this) 
+                        : ColumnCellText.ComputeRowHeight (this);
+                    
+                    header_height = 0;
+                    MoveResize (Allocation);
+                    
+                    RecomputeRowHeight = false;
                 }
                 
                 return row_height;
             }
-            
-            set { row_height = value; }
         }
     }
 }
