@@ -108,7 +108,7 @@ namespace Lastfm
         
         public void UpdateNetworkState (bool connected)
         {
-            Log.DebugFormat ("Changing Audioscrobbler connected state: {0}", connected ? "connected" : "disconnected");
+            Log.DebugFormat ("Audioscrobbler state: {0}", connected ? "connected" : "disconnected");
             this.connected = connected;
         }
 
@@ -169,10 +169,9 @@ namespace Lastfm
 
         private void StateTransitionHandler (object o, ElapsedEventArgs e)
         {
-            Hyena.Log.DebugFormat ("State transition handler running; state: {0}, connected: {1}", state, connected);
+            Log.DebugFormat ("State transition handler running; state: {0}, connected: {1}", state, connected);
             
-            /* if we're not connected, don't bother doing anything
-             * involving the network. */
+            // if we're not connected, don't bother doing anything involving the network.
             if (!connected) {
                 return;
             }
@@ -182,9 +181,7 @@ namespace Lastfm
                 hard_failures = 0;
             }
             
-            
-
-            /* and address changes in our engine state */
+            // and address changes in our engine state
             switch (state) {
             case State.Idle:
                 if (LastfmCore.Account.UserName != null &&
@@ -218,7 +215,7 @@ namespace Lastfm
             case State.WaitingForResponse:
             case State.WaitingForRequestStream:
             case State.WaitingForHandshakeResp:
-                /* nothing here */
+                // nothing here
                 break;
             }
         }
@@ -236,9 +233,9 @@ namespace Lastfm
         {
             int num_tracks_transmitted;
 
-            /* save here in case we're interrupted before we complete
-             * the request.  we save it again when we get an OK back
-             * from the server */
+            // save here in case we're interrupted before we complete
+            // the request.  we save it again when we get an OK back
+            // from the server
             queue.Save ();
 
             next_interval = DateTime.MinValue;
@@ -317,7 +314,7 @@ namespace Lastfm
                 resp = current_web_req.EndGetResponse (ar);
             }
             catch (Exception e) {
-                Hyena.Log.Warning (String.Format("Failed to get the response: {0}", e));
+                Hyena.Log.Warning (String.Format("Failed to get the response: {0}", e), false);
 
                 state = State.Idle;
                 next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
@@ -340,7 +337,7 @@ namespace Lastfm
                     last_upload_failed_logged = now;
                 }
                 
-                /* retransmit the queue on the next interval */
+                // retransmit the queue on the next interval
                 hard_failures++;
                 state = State.NeedTransmit;
             }
@@ -350,7 +347,7 @@ namespace Lastfm
                     last_upload_failed_logged = now;
                 }
                 
-                /* attempt to re-handshake (and retransmit) on the next interval */
+                // attempt to re-handshake (and retransmit) on the next interval
                 session_id = null;
                 next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
                 state = State.NeedHandshake;
@@ -364,14 +361,14 @@ namespace Lastfm
                 
                 hard_failures = 0;
                 
-                /* we succeeded, pop the elements off our queue */
+                // we succeeded, pop the elements off our queue
                 queue.RemoveRange (0, ts.Count);
                 queue.Save ();
                 
                 state = State.Idle;
             } else {
                 if (now - last_upload_failed_logged > TimeSpan.FromMinutes(FAILURE_LOG_MINUTES)) {
-                    Hyena.Log.Warning ("Audioscrobbler upload failed", String.Format ("Unrecognized response: {0}", line));
+                    Hyena.Log.Warning ("Audioscrobbler upload failed", String.Format ("Unrecognized response: {0}", line), false);
                     last_upload_failed_logged = now;
                 }
                 
@@ -427,7 +424,7 @@ namespace Lastfm
             catch (Exception e) {
                 Hyena.Log.Warning ("Failed to handshake: {0}", e.ToString (), false);
 
-                /* back off for a time before trying again */
+                // back off for a time before trying again
                 state = State.Idle;
                 next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
                 return;
@@ -464,7 +461,7 @@ namespace Lastfm
             }
             
             if (success == true) {
-                Hyena.Log.Debug ("Audioscrobbler sign-on succeeded", "Session ID received"); 
+                Log.Debug ("Audioscrobbler sign-on succeeded", "Session ID received"); 
                 session_id = sr.ReadLine ().Trim ();
                 now_playing_url = sr.ReadLine ().Trim ();
                 post_url = sr.ReadLine ().Trim ();
@@ -581,7 +578,7 @@ namespace Lastfm
                 
                 if (line.StartsWith ("BADSESSION")) {
                     Hyena.Log.Warning ("Audioscrobbler NowPlaying failed", "Session ID sent was invalid", false);
-                    /* attempt to re-handshake on the next interval */
+                    // attempt to re-handshake on the next interval
                     session_id = null;
                     next_interval = DateTime.Now + new TimeSpan (0, 0, RETRY_SECONDS);
                     state = State.NeedHandshake;
